function [engine, loglik] = enter_evidence(engine, evidence, varargin)
% ENTER_EVIDENCE Add the specified evidence to the network (jtree_ndx_dbn)
% [engine, loglik] = enter_evidence(engine, evidence, ...)
%
% evidence{i,t} = [] if if X(i,t) is hidden, and otherwise contains its observed value (scalar or column vector)
%
% The following optional arguments can be specified in the form of name/value pairs:
% [default value in brackets]
%
% maximize - if 1, does max-product instead of sum-product [0]
%
% e.g., engine = enter_evidence(engine, ev, 'maximize', 1)

if ~iscell(evidence), error('evidence must be a cell array'); end
if iscell(evidence) & iscell(evidence{1}),
  error('evidence must be a cell array, not a cell array of cell arrays')
end

[ss T] = size(evidence);
bnet = bnet_from_engine(engine);

obs_bitv = ~isemptycell(evidence(:));
bitv = reshape(obs_bitv, ss, T);
for t=1:T
  onodes = find(bitv(:,t));
  if ~isequal(onodes, bnet.observed(:))
    error(['dbn was created assuming observed nodes per slice were '...
	   num2str(bnet.observed(:)')  ' but the evidence in slice ' num2str(t) ...
	   ' has observed nodes ' num2str(onodes(:)')]);
  end
end

% for add_ev in marginal_nodes
engine.evidence = evidence;
engine.T = T;
ns = bnet.node_sizes_slice(:);
engine.actual_node_sizes = repmat(ns, [1 T]);
ns(bnet.observed) = 1;
engine.eff_node_sizes = repmat(ns, [1 T]);

maximize = 0;
filter = 0;

% parse optional params
args = varargin;
nargs = length(args);
if nargs > 0
  for i=1:2:nargs
    switch args{i},
     case 'maximize', maximize = args{i+1}; 
     case 'filter', filter = args{i+1}; 
     otherwise,  
      error(['invalid argument name ' args{i}]);       
    end
  end
end

assert(~filter);
engine.maximize = maximize;
engine.jtree_engine = set_fields(engine.jtree_engine, 'maximize', maximize);
engine.jtree_engine1 = set_fields(engine.jtree_engine1, 'maximize', maximize);


CPDpot = convert_dbn_CPDs_to_tables(bnet, evidence); 
%CPDpot = convert_dbn_CPDs_to_tables_slow(bnet, evidence); 


[engine.clpot, loglik] = enter_soft_evidence(engine, CPDpot);
