function marginal = marginal_nodes(engine, query, t, add_ev)
% MARGINAL_NODES Compute the marginal on the specified query nodes (jtree_ndx_dbn)
%
%   marginal = marginal_nodes(engine, i, t)
% returns Pr(X(i,t) | Y(1:T)), where X(i,t) is the i'th node in the t'th slice.
%
% marginal = marginal_nodes(engine, i, t, add_ev)
% add_ev is an optional argument; if 1, we will "inflate" the marginal of observed nodes
% to their original size, adding 0s to the positions which contradict the evidence
   
if nargin < 4, add_ev = 0; end

if length(query) ~= 1
  error('can only compute marginals on single nodes or families')
end

bnet = bnet_from_engine(engine);
ss = length(bnet.intra);

if isempty(engine.T), error('must call enter_evidence before marginal_family'), end
if t==engine.T
  J = engine.jtree_struct1;
else
  J = engine.jtree_struct;
end
c = J.clq_ass_to_node(query);
id = J.marg_node_ndx_id(query);
ndx = get_ndx(id, engine.ndx_type);

marginal.domain = query + (t-1)*ss;
marginal.T = marg_table_ndx(engine.clpot{c,t}, engine.maximize, ndx, engine.ndx_type);

if add_ev
  marginal = add_ev_to_dmarginal(marginal, engine.evidence, engine.actual_node_sizes(:));
end

