function [clpot, loglik] = enter_soft_evidence(engine, clique, potential, onodes, pot_type)
% ENTER_SOFT_EVIDENCE Add the specified potentials to the network (jtree)
% [clpot, loglik] = enter_soft_evidence(engine, clique, potential, onodes, pot_type, maximize)
%
% We multiply potential{i} onto clique(i) before propagating.
% We return all the modified clique potentials.

bnet = bnet_from_engine(engine);
ns = bnet.node_sizes(:);
cnodes = bnet.cnodes;

% Set the clique potentials to all 1s
C = length(engine.cliques);
clpot = cell(1, C);
for i=1:C
  clpot{i} = mk_initial_pot(pot_type, engine.cliques{i}, ns, cnodes, onodes);
end

% Multiply on specified potentials
for i=1:length(clique)
  c = clique(i);
  clpot{c} = multiply_by_pot(clpot{c}, potential{i});
end

seppot = cell(C, C);
% separators are implicitely initialized to 1s
% They are only used locally within this function

[clpot, seppot] = collect_evidence(engine, clpot, seppot);
[clpot, seppot] = distribute_evidence(engine, clpot, seppot);

ll = zeros(1, C);
for i=1:C
  [clpot{i}, ll(i)] = normalize_pot(clpot{i});
end
loglik = ll(1); % we can extract the likelihood from any clique


