function [clpot, seppot] = init_pot(engine, nodes, CPDs, clqs, pots, ndx2)
% INIT_POT Initialise potentials with evidence (jtree_ndx_inf)
% function [clpot, seppot] = init_pot(engine, nodes, CPDs, clqs, pots, ndx)
%
% CPDs are the potentials for nodes, and will use the built-in indices.
% clqs, pots, ndx are optional extra cliques, potentials and indices.


cliques = engine.cliques;
ns = engine.eff_node_sizes;

% Set the clique potentials to all 1s
C = length(cliques);
clpot = cell(1,C);
for i=1:C
  clpot{i} = ones(prod(ns(cliques{i})), 1);
end

ndx_type = engine.ndx_type;

% Multiply on specified potentials
for i=1:length(nodes)
  n = nodes(i);
  c = engine.clq_ass_to_node(n);
  ndx = get_ndx(engine.mult_node_ndx_id(n), ndx_type);
  clpot{c} = mult_by_table_ndx(clpot{c}, CPDs{i}, ndx, ndx_type);
end

if nargin > 3
  for i=1:length(clqs)
    c = clqs(i);
    clpot{c} = mult_by_table_ndx(clpot{c}, pots{i}, ndx2{i}, ndx_type);
  end
end

seppot = cell(C,C); % implicitely initialized to 1

